package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.Block;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.parser.Parser;

public class Initializer extends FieldDeclaration {
  public Block block;
  
  public int lastVisibleFieldID;
  
  public int bodyStart;
  
  public int bodyEnd;
  
  private MethodBinding methodBinding;
  
  public Initializer(Block block, int modifiers) {
    this.block = block;
    this.modifiers = modifiers;
    if (block != null)
      this.declarationSourceStart = this.sourceStart = block.sourceStart; 
  }
  
  public FlowInfo analyseCode(MethodScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    if (this.block != null)
      return this.block.analyseCode((BlockScope)currentScope, flowContext, flowInfo); 
    return flowInfo;
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream) {
    if ((this.bits & Integer.MIN_VALUE) == 0)
      return; 
    int pc = codeStream.position;
    if (this.block != null)
      this.block.generateCode(currentScope, codeStream); 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public int getKind() {
    return 2;
  }
  
  public boolean isStatic() {
    return ((this.modifiers & 0x8) != 0);
  }
  
  public void parseStatements(Parser parser, TypeDeclaration typeDeclaration, CompilationUnitDeclaration unit) {
    parser.parse(this, typeDeclaration, unit);
  }
  
  public StringBuffer printStatement(int indent, StringBuffer output) {
    if (this.modifiers != 0) {
      printIndent(indent, output);
      printModifiers(this.modifiers, output);
      if (this.annotations != null) {
        printAnnotations(this.annotations, output);
        output.append(' ');
      } 
      output.append("{\n");
      if (this.block != null)
        this.block.printBody(indent, output); 
      printIndent(indent, output).append('}');
      return output;
    } 
    if (this.block != null) {
      this.block.printStatement(indent, output);
    } else {
      printIndent(indent, output).append("{}");
    } 
    return output;
  }
  
  public void resolve(MethodScope scope) {
    FieldBinding previousField = scope.initializedField;
    int previousFieldID = scope.lastVisibleFieldID;
    try {
      scope.initializedField = null;
      scope.lastVisibleFieldID = this.lastVisibleFieldID;
      if (isStatic()) {
        SourceTypeBinding sourceTypeBinding = scope.enclosingSourceType();
        if (sourceTypeBinding.isNestedType() && !sourceTypeBinding.isStatic())
          scope.problemReporter().innerTypesCannotDeclareStaticInitializers(
              (ReferenceBinding)sourceTypeBinding, 
              this); 
      } 
      if (this.block != null)
        this.block.resolve((BlockScope)scope); 
    } finally {
      scope.initializedField = previousField;
      scope.lastVisibleFieldID = previousFieldID;
    } 
  }
  
  public MethodBinding getMethodBinding() {
    if (this.methodBinding == null) {
      BlockScope blockScope = this.block.scope;
      this.methodBinding = isStatic() ? 
        new MethodBinding(8, CharOperation.NO_CHAR, (TypeBinding)TypeBinding.VOID, Binding.NO_PARAMETERS, Binding.NO_EXCEPTIONS, (ReferenceBinding)blockScope.enclosingSourceType()) : 
        new MethodBinding(0, CharOperation.NO_CHAR, (TypeBinding)TypeBinding.VOID, Binding.NO_PARAMETERS, Binding.NO_EXCEPTIONS, (ReferenceBinding)blockScope.enclosingSourceType());
    } 
    return this.methodBinding;
  }
  
  public void traverse(ASTVisitor visitor, MethodScope scope) {
    if (visitor.visit(this, scope) && 
      this.block != null)
      this.block.traverse(visitor, (BlockScope)scope); 
    visitor.endVisit(this, scope);
  }
}
